@@ -211,15 +211,15 @@ class HTTPConnectionTest(AsyncHTTPTestCase):
             b"X-Header-encoding-test: \xe9",
         ],
             b"\r\n".join([
-            b"Content-Disposition: form-data; name=argument",
-            b"",
-            u("\u00e1").encode("utf-8"),
-            b"--1234567890",
-            u('Content-Disposition: form-data; name="files"; filename="\u00f3"').encode("utf8"),
-            b"",
-            u("\u00fa").encode("utf-8"),
-            b"--1234567890--",
-            b"",
+                b"Content-Disposition: form-data; name=argument",
+                b"",
+                u("\u00e1").encode("utf-8"),
+                b"--1234567890",
+                u('Content-Disposition: form-data; name="files"; filename="\u00f3"').encode("utf8"),
+                b"",
+                u("\u00fa").encode("utf-8"),
+                b"--1234567890--",
+                b"",
             ]))
         data = json_decode(response.body)
         self.assertEqual(u("\u00e9"), data["header"])
@@ -344,6 +344,21 @@ class HTTPServerTest(AsyncHTTPTestCase):
         self.assertEqual(200, response.code)
         self.assertEqual(json_decode(response.body), {})
 
+    def test_malformed_body(self):
+        # parse_qs is pretty forgiving, but it will fail on python 3
+        # if the data is not utf8.  On python 2 parse_qs will work,
+        # but then the recursive_unicode call in EchoHandler will
+        # fail.
+        if str is bytes_type:
+            return
+        with ExpectLog(gen_log, 'Invalid x-www-form-urlencoded body'):
+            response = self.fetch(
+                '/echo', method="POST",
+                headers={'Content-Type': 'application/x-www-form-urlencoded'},
+                body=b'\xe9')
+        self.assertEqual(200, response.code)
+        self.assertEqual(b'{}', response.body)
+
 
 class HTTPServerRawTest(AsyncHTTPTestCase):
     def get_app(self):
@@ -497,31 +512,40 @@ class UnixSocketTest(AsyncTestCase):
     def setUp(self):
         super(UnixSocketTest, self).setUp()
         self.tmpdir = tempfile.mkdtemp()
+        self.sockfile = os.path.join(self.tmpdir, "test.sock")
+        sock = netutil.bind_unix_socket(self.sockfile)
+        app = Application([("/hello", HelloWorldRequestHandler)])
+        self.server = HTTPServer(app, io_loop=self.io_loop)
+        self.server.add_socket(sock)
+        self.stream = IOStream(socket.socket(socket.AF_UNIX), io_loop=self.io_loop)
+        self.stream.connect(self.sockfile, self.stop)
+        self.wait()
 
     def tearDown(self):
+        self.stream.close()
+        self.server.stop()
         shutil.rmtree(self.tmpdir)
         super(UnixSocketTest, self).tearDown()
 
     def test_unix_socket(self):
-        sockfile = os.path.join(self.tmpdir, "test.sock")
-        sock = netutil.bind_unix_socket(sockfile)
-        app = Application([("/hello", HelloWorldRequestHandler)])
-        server = HTTPServer(app, io_loop=self.io_loop)
-        server.add_socket(sock)
-        stream = IOStream(socket.socket(socket.AF_UNIX), io_loop=self.io_loop)
-        stream.connect(sockfile, self.stop)
-        self.wait()
-        stream.write(b"GET /hello HTTP/1.0\r\n\r\n")
-        stream.read_until(b"\r\n", self.stop)
+        self.stream.write(b"GET /hello HTTP/1.0\r\n\r\n")
+        self.stream.read_until(b"\r\n", self.stop)
         response = self.wait()
         self.assertEqual(response, b"HTTP/1.0 200 OK\r\n")
-        stream.read_until(b"\r\n\r\n", self.stop)
+        self.stream.read_until(b"\r\n\r\n", self.stop)
         headers = HTTPHeaders.parse(self.wait().decode('latin1'))
-        stream.read_bytes(int(headers["Content-Length"]), self.stop)
+        self.stream.read_bytes(int(headers["Content-Length"]), self.stop)
         body = self.wait()
         self.assertEqual(body, b"Hello world")
-        stream.close()
-        server.stop()
+
+    def test_unix_socket_bad_request(self):
+        # Unix sockets don't have remote addresses so they just return an
+        # empty string.
+        with ExpectLog(gen_log, "Malformed HTTP request from"):
+            self.stream.write(b"garbage\r\n\r\n")
+            self.stream.read_until_close(self.stop)
+            response = self.wait()
+        self.assertEqual(response, b"")
 
 
 class KeepAliveTest(AsyncHTTPTestCase):
