@@ -9,7 +9,7 @@ from tornado.template import DictLoader
 from tornado.testing import AsyncHTTPTestCase, ExpectLog
 from tornado.test.util import unittest
 from tornado.util import u, bytes_type, ObjectDict, unicode_type
-from tornado.web import RequestHandler, authenticated, Application, asynchronous, url, HTTPError, StaticFileHandler, _create_signature, create_signed_value, ErrorHandler, UIModule, MissingArgumentError
+from tornado.web import RequestHandler, authenticated, Application, asynchronous, url, HTTPError, StaticFileHandler, _create_signature_v1, create_signed_value, decode_signed_value, ErrorHandler, UIModule, MissingArgumentError
 
 import binascii
 import datetime
@@ -20,6 +20,11 @@ import re
 import socket
 import sys
 
+try:
+    import urllib.parse as urllib_parse  # py3
+except ImportError:
+    import urllib as urllib_parse  # py2
+
 wsgi_safe_tests = []
 
 relpath = lambda *a: os.path.join(os.path.dirname(__file__), *a)
@@ -56,6 +61,11 @@ class SimpleHandlerTestCase(WebTestCase):
         return [('/', self.Handler)]
 
 
+class HelloHandler(RequestHandler):
+    def get(self):
+        self.write('hello')
+
+
 class CookieTestRequestHandler(RequestHandler):
     # stub out enough methods to make the secure_cookie functions work
     def __init__(self):
@@ -70,30 +80,33 @@ class CookieTestRequestHandler(RequestHandler):
         self._cookies[name] = value
 
 
-class SecureCookieTest(unittest.TestCase):
+# See SignedValueTest below for more.
+class SecureCookieV1Test(unittest.TestCase):
     def test_round_trip(self):
         handler = CookieTestRequestHandler()
-        handler.set_secure_cookie('foo', b'bar')
-        self.assertEqual(handler.get_secure_cookie('foo'), b'bar')
+        handler.set_secure_cookie('foo', b'bar', version=1)
+        self.assertEqual(handler.get_secure_cookie('foo', min_version=1),
+                         b'bar')
 
     def test_cookie_tampering_future_timestamp(self):
         handler = CookieTestRequestHandler()
         # this string base64-encodes to '12345678'
-        handler.set_secure_cookie('foo', binascii.a2b_hex(b'd76df8e7aefc'))
+        handler.set_secure_cookie('foo', binascii.a2b_hex(b'd76df8e7aefc'),
+                                  version=1)
         cookie = handler._cookies['foo']
         match = re.match(br'12345678\|([0-9]+)\|([0-9a-f]+)', cookie)
         self.assertTrue(match)
         timestamp = match.group(1)
         sig = match.group(2)
         self.assertEqual(
-            _create_signature(handler.application.settings["cookie_secret"],
+            _create_signature_v1(handler.application.settings["cookie_secret"],
                               'foo', '12345678', timestamp),
             sig)
         # shifting digits from payload to timestamp doesn't alter signature
         # (this is not desirable behavior, just confirming that that's how it
         # works)
         self.assertEqual(
-            _create_signature(handler.application.settings["cookie_secret"],
+            _create_signature_v1(handler.application.settings["cookie_secret"],
                               'foo', '1234', b'5678' + timestamp),
             sig)
         # tamper with the cookie
@@ -101,14 +114,15 @@ class SecureCookieTest(unittest.TestCase):
             to_basestring(timestamp), to_basestring(sig)))
         # it gets rejected
         with ExpectLog(gen_log, "Cookie timestamp in future"):
-            self.assertTrue(handler.get_secure_cookie('foo') is None)
+            self.assertTrue(
+                handler.get_secure_cookie('foo', min_version=1) is None)
 
     def test_arbitrary_bytes(self):
         # Secure cookies accept arbitrary data (which is base64 encoded).
         # Note that normal cookies accept only a subset of ascii.
         handler = CookieTestRequestHandler()
-        handler.set_secure_cookie('foo', b'\xe9')
-        self.assertEqual(handler.get_secure_cookie('foo'), b'\xe9')
+        handler.set_secure_cookie('foo', b'\xe9', version=1)
+        self.assertEqual(handler.get_secure_cookie('foo', min_version=1), b'\xe9')
 
 
 class CookieTest(WebTestCase):
@@ -479,8 +493,21 @@ class HeaderInjectionHandler(RequestHandler):
 
 
 class GetArgumentHandler(RequestHandler):
-    def get(self):
-        self.write(self.get_argument("foo", "default"))
+    def prepare(self):
+        if self.get_argument('source', None) == 'query':
+            method = self.get_query_argument
+        elif self.get_argument('source', None) == 'body':
+            method = self.get_body_argument
+        else:
+            method = self.get_argument
+        self.finish(method("foo", "default"))
+
+
+class GetArgumentsHandler(RequestHandler):
+    def prepare(self):
+        self.finish(dict(default=self.get_arguments("foo"),
+                         query=self.get_query_arguments("foo"),
+                         body=self.get_body_arguments("foo")))
 
 
 # This test is shared with wsgi_test.py
@@ -521,6 +548,7 @@ class WSGISafeWebTest(WebTestCase):
             url("/redirect", RedirectHandler),
             url("/header_injection", HeaderInjectionHandler),
             url("/get_argument", GetArgumentHandler),
+            url("/get_arguments", GetArgumentsHandler),
         ]
         return urls
 
@@ -562,6 +590,14 @@ class WSGISafeWebTest(WebTestCase):
                                 u('query'): [u('bytes'), u('c3a9')],
                                 })
 
+    def test_decode_argument_invalid_unicode(self):
+        # test that invalid unicode in URLs causes 400, not 500
+        with ExpectLog(gen_log, ".*Invalid unicode.*"):
+            response = self.fetch("/typecheck/invalid%FF")
+            self.assertEqual(response.code, 400)
+            response = self.fetch("/typecheck/invalid?foo=%FF")
+            self.assertEqual(response.code, 400)
+
     def test_decode_argument_plus(self):
         # These urls are all equivalent.
         urls = ["/decode_arg/1%20%2B%201?foo=1%20%2B%201&encoding=utf-8",
@@ -647,6 +683,49 @@ js_embed()
         response = self.fetch("/get_argument")
         self.assertEqual(response.body, b"default")
 
+        # Test merging of query and body arguments.
+        # In singular form, body arguments take precedence over query arguments.
+        body = urllib_parse.urlencode(dict(foo="hello"))
+        response = self.fetch("/get_argument?foo=bar", method="POST", body=body)
+        self.assertEqual(response.body, b"hello")
+        # In plural methods they are merged.
+        response = self.fetch("/get_arguments?foo=bar",
+                              method="POST", body=body)
+        self.assertEqual(json_decode(response.body),
+                         dict(default=['bar', 'hello'],
+                              query=['bar'],
+                              body=['hello']))
+
+    def test_get_query_arguments(self):
+        # send as a post so we can ensure the separation between query
+        # string and body arguments.
+        body = urllib_parse.urlencode(dict(foo="hello"))
+        response = self.fetch("/get_argument?source=query&foo=bar",
+                              method="POST", body=body)
+        self.assertEqual(response.body, b"bar")
+        response = self.fetch("/get_argument?source=query&foo=",
+                              method="POST", body=body)
+        self.assertEqual(response.body, b"")
+        response = self.fetch("/get_argument?source=query",
+                              method="POST", body=body)
+        self.assertEqual(response.body, b"default")
+
+    def test_get_body_arguments(self):
+        body = urllib_parse.urlencode(dict(foo="bar"))
+        response = self.fetch("/get_argument?source=body&foo=hello",
+                              method="POST", body=body)
+        self.assertEqual(response.body, b"bar")
+
+        body = urllib_parse.urlencode(dict(foo=""))
+        response = self.fetch("/get_argument?source=body&foo=hello",
+                              method="POST", body=body)
+        self.assertEqual(response.body, b"")
+
+        body = urllib_parse.urlencode(dict())
+        response = self.fetch("/get_argument?source=body&foo=hello",
+                              method="POST", body=body)
+        self.assertEqual(response.body, b"default")
+
     def test_no_gzip(self):
         response = self.fetch('/get_argument')
         self.assertNotIn('Accept-Encoding', response.headers.get('Vary', ''))
@@ -1479,6 +1558,91 @@ class SetCurrentUserTest(SimpleHandlerTestCase):
         self.assertEqual(response.body, b'Hello Ben')
 
 
+@wsgi_safe
+class GetCurrentUserTest(WebTestCase):
+    def get_app_kwargs(self):
+        class WithoutUserModule(UIModule):
+            def render(self):
+                return ''
+
+        class WithUserModule(UIModule):
+            def render(self):
+                return str(self.current_user)
+
+        loader = DictLoader({
+            'without_user.html': '',
+            'with_user.html': '{{ current_user }}',
+            'without_user_module.html': '{% module WithoutUserModule() %}',
+            'with_user_module.html': '{% module WithUserModule() %}',
+        })
+        return dict(template_loader=loader,
+                    ui_modules={'WithUserModule': WithUserModule,
+                                'WithoutUserModule': WithoutUserModule})
+
+    def tearDown(self):
+        super(GetCurrentUserTest, self).tearDown()
+        RequestHandler._template_loaders.clear()
+
+    def get_handlers(self):
+        class CurrentUserHandler(RequestHandler):
+            def prepare(self):
+                self.has_loaded_current_user = False
+
+            def get_current_user(self):
+                self.has_loaded_current_user = True
+                return ''
+
+        class WithoutUserHandler(CurrentUserHandler):
+            def get(self):
+                self.render_string('without_user.html')
+                self.finish(str(self.has_loaded_current_user))
+
+        class WithUserHandler(CurrentUserHandler):
+            def get(self):
+                self.render_string('with_user.html')
+                self.finish(str(self.has_loaded_current_user))
+
+        class CurrentUserModuleHandler(CurrentUserHandler):
+            def get_template_namespace(self):
+                # If RequestHandler.get_template_namespace is called, then
+                # get_current_user is evaluated. Until #820 is fixed, this
+                # is a small hack to circumvent the issue.
+                return self.ui
+
+        class WithoutUserModuleHandler(CurrentUserModuleHandler):
+            def get(self):
+                self.render_string('without_user_module.html')
+                self.finish(str(self.has_loaded_current_user))
+
+        class WithUserModuleHandler(CurrentUserModuleHandler):
+            def get(self):
+                self.render_string('with_user_module.html')
+                self.finish(str(self.has_loaded_current_user))
+
+        return [('/without_user', WithoutUserHandler),
+                ('/with_user', WithUserHandler),
+                ('/without_user_module', WithoutUserModuleHandler),
+                ('/with_user_module', WithUserModuleHandler)]
+
+    @unittest.skip('needs fix')
+    def test_get_current_user_is_lazy(self):
+        # TODO: Make this test pass. See #820.
+        response = self.fetch('/without_user')
+        self.assertEqual(response.body, b'False')
+
+    def test_get_current_user_works(self):
+        response = self.fetch('/with_user')
+        self.assertEqual(response.body, b'True')
+
+    def test_get_current_user_from_ui_module_is_lazy(self):
+        response = self.fetch('/without_user_module')
+        self.assertEqual(response.body, b'False')
+
+    def test_get_current_user_from_ui_module_works(self):
+        response = self.fetch('/with_user_module')
+        self.assertEqual(response.body, b'True')
+
+
 @wsgi_safe
 class UnimplementedHTTPMethodsTest(SimpleHandlerTestCase):
     class Handler(RequestHandler):
@@ -1568,3 +1732,321 @@ class FinishInPrepareTest(SimpleHandlerTestCase):
     def test_finish_in_prepare(self):
         response = self.fetch('/')
         self.assertEqual(response.body, b'done')
+
+
+@wsgi_safe
+class Default404Test(WebTestCase):
+    def get_handlers(self):
+        # If there are no handlers at all a default redirect handler gets added.
+        return [('/foo', RequestHandler)]
+
+    def test_404(self):
+        response = self.fetch('/')
+        self.assertEqual(response.code, 404)
+        self.assertEqual(response.body,
+                         b'<html><title>404: Not Found</title>'
+                         b'<body>404: Not Found</body></html>')
+
+
+@wsgi_safe
+class Custom404Test(WebTestCase):
+    def get_handlers(self):
+        return [('/foo', RequestHandler)]
+
+    def get_app_kwargs(self):
+        class Custom404Handler(RequestHandler):
+            def get(self):
+                self.set_status(404)
+                self.write('custom 404 response')
+
+        return dict(default_handler_class=Custom404Handler)
+
+    def test_404(self):
+        response = self.fetch('/')
+        self.assertEqual(response.code, 404)
+        self.assertEqual(response.body, b'custom 404 response')
+
+
+@wsgi_safe
+class DefaultHandlerArgumentsTest(WebTestCase):
+    def get_handlers(self):
+        return [('/foo', RequestHandler)]
+
+    def get_app_kwargs(self):
+        return dict(default_handler_class=ErrorHandler,
+                    default_handler_args=dict(status_code=403))
+
+    def test_403(self):
+        response = self.fetch('/')
+        self.assertEqual(response.code, 403)
+
+
+@wsgi_safe
+class HandlerByNameTest(WebTestCase):
+    def get_handlers(self):
+        # All three are equivalent.
+        return [('/hello1', HelloHandler),
+                ('/hello2', 'tornado.test.web_test.HelloHandler'),
+                url('/hello3', 'tornado.test.web_test.HelloHandler'),
+                ]
+
+    def test_handler_by_name(self):
+        resp = self.fetch('/hello1')
+        self.assertEqual(resp.body, b'hello')
+        resp = self.fetch('/hello2')
+        self.assertEqual(resp.body, b'hello')
+        resp = self.fetch('/hello3')
+        self.assertEqual(resp.body, b'hello')
+
+
+class SignedValueTest(unittest.TestCase):
+    SECRET = "It's a secret to everybody"
+
+    def past(self):
+        return self.present() - 86400 * 32
+
+    def present(self):
+        return 1300000000
+
+    def test_known_values(self):
+        signed_v1 = create_signed_value(SignedValueTest.SECRET, "key", "value",
+                                        version=1, clock=self.present)
+        self.assertEqual(
+            signed_v1,
+            b"dmFsdWU=|1300000000|31c934969f53e48164c50768b40cbd7e2daaaa4f")
+
+        signed_v2 = create_signed_value(SignedValueTest.SECRET, "key", "value",
+                                        version=2, clock=self.present)
+        self.assertEqual(
+            signed_v2,
+            b"2|1:0|10:1300000000|3:key|8:dmFsdWU=|"
+            b"3d4e60b996ff9c5d5788e333a0cba6f238a22c6c0f94788870e1a9ecd482e152")
+
+        signed_default = create_signed_value(SignedValueTest.SECRET,
+                                             "key", "value", clock=self.present)
+        self.assertEqual(signed_default, signed_v2)
+
+        decoded_v1 = decode_signed_value(SignedValueTest.SECRET, "key",
+                                         signed_v1, min_version=1,
+                                         clock=self.present)
+        self.assertEqual(decoded_v1, b"value")
+
+        decoded_v2 = decode_signed_value(SignedValueTest.SECRET, "key",
+                                         signed_v2, min_version=2,
+                                         clock=self.present)
+        self.assertEqual(decoded_v2, b"value")
+
+    def test_name_swap(self):
+        signed1 = create_signed_value(SignedValueTest.SECRET, "key1", "value",
+                                      clock=self.present)
+        signed2 = create_signed_value(SignedValueTest.SECRET, "key2", "value",
+                                      clock=self.present)
+        # Try decoding each string with the other's "name"
+        decoded1 = decode_signed_value(SignedValueTest.SECRET, "key2", signed1,
+                                       clock=self.present)
+        self.assertIs(decoded1, None)
+        decoded2 = decode_signed_value(SignedValueTest.SECRET, "key1", signed2,
+                                       clock=self.present)
+        self.assertIs(decoded2, None)
+
+    def test_expired(self):
+        signed = create_signed_value(SignedValueTest.SECRET, "key1", "value",
+                                     clock=self.past)
+        decoded_past = decode_signed_value(SignedValueTest.SECRET, "key1",
+                                           signed, clock=self.past)
+        self.assertEqual(decoded_past, b"value")
+        decoded_present = decode_signed_value(SignedValueTest.SECRET, "key1",
+                                              signed, clock=self.present)
+        self.assertIs(decoded_present, None)
+
+    def test_payload_tampering(self):
+        # These cookies are variants of the one in test_known_values.
+        sig = "3d4e60b996ff9c5d5788e333a0cba6f238a22c6c0f94788870e1a9ecd482e152"
+        def validate(prefix):
+            return (b'value' ==
+                    decode_signed_value(SignedValueTest.SECRET, "key",
+                                        prefix + sig, clock=self.present))
+        self.assertTrue(validate("2|1:0|10:1300000000|3:key|8:dmFsdWU=|"))
+        # Change key version
+        self.assertFalse(validate("2|1:1|10:1300000000|3:key|8:dmFsdWU=|"))
+        # length mismatch (field too short)
+        self.assertFalse(validate("2|1:0|10:130000000|3:key|8:dmFsdWU=|"))
+        # length mismatch (field too long)
+        self.assertFalse(validate("2|1:0|10:1300000000|3:keey|8:dmFsdWU=|"))
+
+    def test_signature_tampering(self):
+        prefix = "2|1:0|10:1300000000|3:key|8:dmFsdWU=|"
+        def validate(sig):
+            return (b'value' ==
+                    decode_signed_value(SignedValueTest.SECRET, "key",
+                                        prefix + sig, clock=self.present))
+        self.assertTrue(validate(
+            "3d4e60b996ff9c5d5788e333a0cba6f238a22c6c0f94788870e1a9ecd482e152"))
+        # All zeros
+        self.assertFalse(validate("0" * 32))
+        # Change one character
+        self.assertFalse(validate(
+            "4d4e60b996ff9c5d5788e333a0cba6f238a22c6c0f94788870e1a9ecd482e152"))
+        # Change another character
+        self.assertFalse(validate(
+            "3d4e60b996ff9c5d5788e333a0cba6f238a22c6c0f94788870e1a9ecd482e153"))
+        # Truncate
+        self.assertFalse(validate(
+            "3d4e60b996ff9c5d5788e333a0cba6f238a22c6c0f94788870e1a9ecd482e15"))
+        # Lengthen
+        self.assertFalse(validate(
+            "3d4e60b996ff9c5d5788e333a0cba6f238a22c6c0f94788870e1a9ecd482e1538"))
+
+    def test_non_ascii(self):
+        value = b"\xe9"
+        signed = create_signed_value(SignedValueTest.SECRET, "key", value,
+                                     clock=self.present)
+        decoded = decode_signed_value(SignedValueTest.SECRET, "key", signed,
+                                      clock=self.present)
+        self.assertEqual(value, decoded)
+
+
+@wsgi_safe
+class XSRFTest(SimpleHandlerTestCase):
+    class Handler(RequestHandler):
+        def get(self):
+            version = int(self.get_argument("version", "2"))
+            # This would be a bad idea in a real app, but in this test
+            # it's fine.
+            self.settings["xsrf_cookie_version"] = version
+            self.write(self.xsrf_token)
+
+        def post(self):
+            self.write("ok")
+
+    def get_app_kwargs(self):
+        return dict(xsrf_cookies=True)
+
+    def setUp(self):
+        super(XSRFTest, self).setUp()
+        self.xsrf_token = self.get_token()
+
+    def get_token(self, old_token=None, version=None):
+        if old_token is not None:
+            headers = self.cookie_headers(old_token)
+        else:
+            headers = None
+        response = self.fetch(
+            "/" if version is None else ("/?version=%d" % version),
+            headers=headers)
+        response.rethrow()
+        return native_str(response.body)
+
+    def cookie_headers(self, token=None):
+        if token is None:
+            token = self.xsrf_token
+        return {"Cookie": "_xsrf=" + token}
+
+    def test_xsrf_fail_no_token(self):
+        with ExpectLog(gen_log, ".*'_xsrf' argument missing"):
+            response = self.fetch("/", method="POST", body=b"")
+        self.assertEqual(response.code, 403)
+
+    def test_xsrf_fail_body_no_cookie(self):
+        with ExpectLog(gen_log, ".*XSRF cookie does not match POST"):
+            response = self.fetch(
+                "/", method="POST",
+                body=urllib_parse.urlencode(dict(_xsrf=self.xsrf_token)))
+        self.assertEqual(response.code, 403)
+
+    def test_xsrf_fail_cookie_no_body(self):
+        with ExpectLog(gen_log, ".*'_xsrf' argument missing"):
+            response = self.fetch(
+                "/", method="POST", body=b"",
+                headers=self.cookie_headers())
+        self.assertEqual(response.code, 403)
+
+    def test_xsrf_success_post_body(self):
+        response = self.fetch(
+            "/", method="POST",
+            body=urllib_parse.urlencode(dict(_xsrf=self.xsrf_token)),
+            headers=self.cookie_headers())
+        self.assertEqual(response.code, 200)
+
+    def test_xsrf_success_query_string(self):
+        response = self.fetch(
+            "/?" + urllib_parse.urlencode(dict(_xsrf=self.xsrf_token)),
+            method="POST", body=b"",
+            headers=self.cookie_headers())
+        self.assertEqual(response.code, 200)
+
+    def test_xsrf_success_header(self):
+        response = self.fetch("/", method="POST", body=b"",
+                              headers=dict({"X-Xsrftoken": self.xsrf_token},
+                                           **self.cookie_headers()))
+        self.assertEqual(response.code, 200)
+
+    def test_distinct_tokens(self):
+        # Every request gets a distinct token.
+        NUM_TOKENS = 10
+        tokens = set()
+        for i in range(NUM_TOKENS):
+            tokens.add(self.get_token())
+        self.assertEqual(len(tokens), NUM_TOKENS)
+
+    def test_cross_user(self):
+        token2 = self.get_token()
+        # Each token can be used to authenticate its own request.
+        for token in (self.xsrf_token, token2):
+            response  = self.fetch(
+                "/", method="POST",
+                body=urllib_parse.urlencode(dict(_xsrf=token)),
+                headers=self.cookie_headers(token))
+            self.assertEqual(response.code, 200)
+        # Sending one in the cookie and the other in the body is not allowed.
+        for cookie_token, body_token in ((self.xsrf_token, token2),
+                                         (token2, self.xsrf_token)):
+            with ExpectLog(gen_log, '.*XSRF cookie does not match POST'):
+                response = self.fetch(
+                    "/", method="POST",
+                    body=urllib_parse.urlencode(dict(_xsrf=body_token)),
+                    headers=self.cookie_headers(cookie_token))
+            self.assertEqual(response.code, 403)
+
+    def test_refresh_token(self):
+        token = self.xsrf_token
+        tokens_seen = set([token])
+        # A user's token is stable over time.  Refreshing the page in one tab
+        # might update the cookie while an older tab still has the old cookie
+        # in its DOM.  Simulate this scenario by passing a constant token
+        # in the body and re-querying for the token.
+        for i in range(5):
+            token = self.get_token(token)
+            # Tokens are encoded uniquely each time
+            tokens_seen.add(token)
+            response = self.fetch(
+                "/", method="POST",
+                body=urllib_parse.urlencode(dict(_xsrf=self.xsrf_token)),
+                headers=self.cookie_headers(token))
+            self.assertEqual(response.code, 200)
+        self.assertEqual(len(tokens_seen), 6)
+
+    def test_versioning(self):
+        # Version 1 still produces distinct tokens per request.
+        self.assertNotEqual(self.get_token(version=1),
+                            self.get_token(version=1))
+
+        # Refreshed v1 tokens are all identical.
+        v1_token = self.get_token(version=1)
+        for i in range(5):
+            self.assertEqual(self.get_token(v1_token, version=1), v1_token)
+
+        # Upgrade to a v2 version of the same token
+        v2_token = self.get_token(v1_token)
+        self.assertNotEqual(v1_token, v2_token)
+        # Each v1 token can map to many v2 tokens.
+        self.assertNotEqual(v2_token, self.get_token(v1_token))
+
+        # The tokens are cross-compatible.
+        for cookie_token, body_token in ((v1_token, v2_token),
+                                         (v2_token, v1_token)):
+            response = self.fetch(
+                "/", method="POST",
+                body=urllib_parse.urlencode(dict(_xsrf=body_token)),
+                headers=self.cookie_headers(cookie_token))
+            self.assertEqual(response.code, 200)
